@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -22,8 +20,8 @@
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
-/*
- * Copyright (c) 1990, 1993
+/*-
+ * Copyright (c) 1990, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
  *
  * This code is derived from software contributed to Berkeley by
@@ -58,6 +56,10 @@
  * SUCH DAMAGE.
  */
 
+#if defined(LIBC_SCCS) && !defined(lint)
+static char sccsid[] = "@(#)hash_page.c	8.7 (Berkeley) 8/16/94";
+#endif /* LIBC_SCCS and not lint */
+#include <sys/cdefs.h>
 
 /*
  * PACKAGE:  hashing
@@ -93,19 +95,19 @@
 #include "page.h"
 #include "extern.h"
 
-static u_long	*fetch_bitmap __P((HTAB *, int));
-static u_long	 first_free __P((u_long));
-static int	 open_temp __P((HTAB *));
-static u_short	 overflow_page __P((HTAB *));
-static void	 putpair __P((char *, const DBT *, const DBT *));
-static void	 squeeze_key __P((u_short *, const DBT *, const DBT *));
+static u_int32_t	*fetch_bitmap(HTAB *, int);
+static u_int32_t	 first_free(u_int32_t);
+static int	 open_temp(HTAB *);
+static u_int16_t	 overflow_page(HTAB *);
+static void	 putpair(char *, const DBT *, const DBT *);
+static void	 squeeze_key(u_int16_t *, const DBT *, const DBT *);
 static int	 ugly_split
-		    __P((HTAB *, u_int, BUFHEAD *, BUFHEAD *, int, int));
+(HTAB *, u_int32_t, BUFHEAD *, BUFHEAD *, int, int);
 
 #define	PAGE_INIT(P) { \
-	((u_short *)(P))[0] = 0; \
-	((u_short *)(P))[1] = hashp->BSIZE - 3 * sizeof(u_short); \
-	((u_short *)(P))[2] = hashp->BSIZE; \
+	((u_int16_t *)(P))[0] = 0; \
+	((u_int16_t *)(P))[1] = hashp->BSIZE - 3 * sizeof(u_int16_t); \
+	((u_int16_t *)(P))[2] = hashp->BSIZE; \
 }
 
 /*
@@ -118,9 +120,9 @@ putpair(p, key, val)
 	char *p;
 	const DBT *key, *val;
 {
-	register u_short *bp, n, off;
+	u_int16_t *bp, n, off;
 
-	bp = (u_short *)p;
+	bp = (u_int16_t *)p;
 
 	/* Enter the key first. */
 	n = bp[0];
@@ -136,7 +138,7 @@ putpair(p, key, val)
 
 	/* Adjust page info. */
 	bp[0] = n;
-	bp[n + 1] = off - ((n + 3) * sizeof(u_short));
+	bp[n + 1] = off - ((n + 3) * sizeof(u_int16_t));
 	bp[n + 2] = off;
 }
 
@@ -149,13 +151,13 @@ extern int
 __delpair(hashp, bufp, ndx)
 	HTAB *hashp;
 	BUFHEAD *bufp;
-	register int ndx;
+	int ndx;
 {
-	register u_short *bp, newoff;
-	register int n;
-	u_short pairlen;
+	u_int16_t *bp, newoff;
+	int n;
+	u_int16_t pairlen;
 
-	bp = (u_short *)bufp->page;
+	bp = (u_int16_t *)bufp->page;
 	n = bp[0];
 
 	if (bp[ndx + 1] < REAL_KEY)
@@ -168,9 +170,9 @@ __delpair(hashp, bufp, ndx)
 
 	if (ndx != (n - 1)) {
 		/* Hard Case -- need to shuffle keys */
-		register int i;
-		register char *src = bufp->page + (int)OFFSET(bp);
-		register char *dst = src + (int)pairlen;
+		int i;
+		char *src = bufp->page + (int)OFFSET(bp);
+		char *dst = src + (int)pairlen;
 		memmove(dst, src, bp[ndx + 1] - OFFSET(bp));
 
 		/* Now adjust the pointers */
@@ -186,7 +188,7 @@ __delpair(hashp, bufp, ndx)
 	}
 	/* Finally adjust the page data */
 	bp[n] = OFFSET(bp) + pairlen;
-	bp[n - 1] = bp[n + 1] + pairlen + 2 * sizeof(u_short);
+	bp[n - 1] = bp[n + 1] + pairlen + 2 * sizeof(u_int16_t);
 	bp[0] = n - 2;
 	hashp->NKEYS--;
 
@@ -201,18 +203,18 @@ __delpair(hashp, bufp, ndx)
 extern int
 __split_page(hashp, obucket, nbucket)
 	HTAB *hashp;
-	u_int obucket, nbucket;
+	u_int32_t obucket, nbucket;
 {
-	register BUFHEAD *new_bufp, *old_bufp;
-	register u_short *ino;
-	register char *np;
+	BUFHEAD *new_bufp, *old_bufp;
+	u_int16_t *ino;
+	char *np;
 	DBT key, val;
 	int n, ndx, retval;
-	u_short copyto, diff, off, moved;
+	u_int16_t copyto, diff, off, moved;
 	char *op;
 
-	copyto = (u_short)hashp->BSIZE;
-	off = (u_short)hashp->BSIZE;
+	copyto = (u_int16_t)hashp->BSIZE;
+	off = (u_int16_t)hashp->BSIZE;
 	old_bufp = __get_buf(hashp, obucket, NULL, 0);
 	if (old_bufp == NULL)
 		return (-1);
@@ -223,7 +225,7 @@ __split_page(hashp, obucket, nbucket)
 	old_bufp->flags |= (BUF_MOD | BUF_PIN);
 	new_bufp->flags |= (BUF_MOD | BUF_PIN);
 
-	ino = (u_short *)(op = old_bufp->page);
+	ino = (u_int16_t *)(op = old_bufp->page);
 	np = new_bufp->page;
 
 	moved = 0;
@@ -265,13 +267,13 @@ __split_page(hashp, obucket, nbucket)
 
 	/* Now clean up the page */
 	ino[0] -= moved;
-	FREESPACE(ino) = copyto - sizeof(u_short) * (ino[0] + 3);
+	FREESPACE(ino) = copyto - sizeof(u_int16_t) * (ino[0] + 3);
 	OFFSET(ino) = copyto;
 
 #ifdef DEBUG3
 	(void)fprintf(stderr, "split %d/%d\n",
-	    ((u_short *)np)[0] / 2,
-	    ((u_short *)op)[0] / 2);
+	    ((u_int16_t *)np)[0] / 2,
+	    ((u_int16_t *)op)[0] / 2);
 #endif
 	/* unpin both pages */
 	old_bufp->flags &= ~BUF_PIN;
@@ -297,28 +299,28 @@ __split_page(hashp, obucket, nbucket)
 static int
 ugly_split(hashp, obucket, old_bufp, new_bufp, copyto, moved)
 	HTAB *hashp;
-	u_int obucket;	/* Same as __split_page. */
+	u_int32_t obucket;	/* Same as __split_page. */
 	BUFHEAD *old_bufp, *new_bufp;
 	int copyto;	/* First byte on page which contains key/data values. */
-	int moved;	/* Number of pairs moved to new page. */
+	int moved;		/* Number of pairs moved to new page. */
 {
-	register BUFHEAD *bufp;	/* Buffer header for ino */
-	register u_short *ino;	/* Page keys come off of */
-	register u_short *np;	/* New page */
-	register u_short *op;	/* Page keys go on to if they aren't moving */
+	BUFHEAD *bufp;		/* Buffer header for ino */
+	u_int16_t *ino;		/* Page keys come off of */
+	u_int16_t *np;		/* New page */
+	u_int16_t *op;		/* Page keys go on to if they aren't moving */
 
 	BUFHEAD *last_bfp;	/* Last buf header OVFL needing to be freed */
 	DBT key, val;
 	SPLIT_RETURN ret;
-	u_short n, off, ov_addr, scopyto;
+	u_int16_t n, off, ov_addr, scopyto;
 	char *cino;		/* Character value of ino */
 
 	bufp = old_bufp;
-	ino = (u_short *)old_bufp->page;
-	np = (u_short *)new_bufp->page;
-	op = (u_short *)old_bufp->page;
+	ino = (u_int16_t *)old_bufp->page;
+	np = (u_int16_t *)new_bufp->page;
+	op = (u_int16_t *)old_bufp->page;
 	last_bfp = NULL;
-	scopyto = (u_short)copyto;	/* ANSI */
+	scopyto = (u_int16_t)copyto;	/* ANSI */
 
 	n = ino[0] - 1;
 	while (n < ino[0]) {
@@ -329,16 +331,16 @@ ugly_split(hashp, obucket, old_bufp, new_bufp, copyto, moved)
 			old_bufp = ret.oldp;
 			if (!old_bufp)
 				return (-1);
-			op = (u_short *)old_bufp->page;
+			op = (u_int16_t *)old_bufp->page;
 			new_bufp = ret.newp;
 			if (!new_bufp)
 				return (-1);
-			np = (u_short *)new_bufp->page;
+			np = (u_int16_t *)new_bufp->page;
 			bufp = ret.nextp;
 			if (!bufp)
 				return (0);
 			cino = (char *)bufp->page;
-			ino = (u_short *)cino;
+			ino = (u_int16_t *)cino;
 			last_bfp = ret.nextp;
 		} else if (ino[n + 1] == OVFLPAGE) {
 			ov_addr = ino[n];
@@ -348,14 +350,14 @@ ugly_split(hashp, obucket, old_bufp, new_bufp, copyto, moved)
 			 */
 			ino[0] -= (moved + 2);
 			FREESPACE(ino) =
-			    scopyto - sizeof(u_short) * (ino[0] + 3);
+			    scopyto - sizeof(u_int16_t) * (ino[0] + 3);
 			OFFSET(ino) = scopyto;
 
 			bufp = __get_buf(hashp, ov_addr, bufp, 0);
 			if (!bufp)
 				return (-1);
 
-			ino = (u_short *)bufp->page;
+			ino = (u_int16_t *)bufp->page;
 			n = 1;
 			scopyto = hashp->BSIZE;
 			moved = 0;
@@ -383,7 +385,7 @@ ugly_split(hashp, obucket, old_bufp, new_bufp, copyto, moved)
 					    __add_ovflpage(hashp, old_bufp);
 					if (!old_bufp)
 						return (-1);
-					op = (u_short *)old_bufp->page;
+					op = (u_int16_t *)old_bufp->page;
 					putpair((char *)op, &key, &val);
 				}
 				old_bufp->flags |= BUF_MOD;
@@ -396,7 +398,7 @@ ugly_split(hashp, obucket, old_bufp, new_bufp, copyto, moved)
 					    __add_ovflpage(hashp, new_bufp);
 					if (!new_bufp)
 						return (-1);
-					np = (u_short *)new_bufp->page;
+					np = (u_int16_t *)new_bufp->page;
 					putpair((char *)np, &key, &val);
 				}
 				new_bufp->flags |= BUF_MOD;
@@ -421,10 +423,10 @@ __addel(hashp, bufp, key, val)
 	BUFHEAD *bufp;
 	const DBT *key, *val;
 {
-	register u_short *bp, *sop;
+	u_int16_t *bp, *sop;
 	int do_expand;
 
-	bp = (u_short *)bufp->page;
+	bp = (u_int16_t *)bufp->page;
 	do_expand = 0;
 	while (bp[0] && (bp[2] < REAL_KEY || bp[bp[0]] < REAL_KEY))
 		/* Exception case */
@@ -436,7 +438,7 @@ __addel(hashp, bufp, key, val)
 			bufp = __get_buf(hashp, bp[bp[0] - 1], bufp, 0);
 			if (!bufp)
 				return (-1);
-			bp = (u_short *)bufp->page;
+			bp = (u_int16_t *)bufp->page;
 		} else
 			/* Try to squeeze key on this page */
 			if (FREESPACE(bp) > PAIRSIZE(key, val)) {
@@ -446,7 +448,7 @@ __addel(hashp, bufp, key, val)
 				bufp = __get_buf(hashp, bp[bp[0] - 1], bufp, 0);
 				if (!bufp)
 					return (-1);
-				bp = (u_short *)bufp->page;
+				bp = (u_int16_t *)bufp->page;
 			}
 
 	if (PAIRFITS(bp, key, val))
@@ -456,7 +458,7 @@ __addel(hashp, bufp, key, val)
 		bufp = __add_ovflpage(hashp, bufp);
 		if (!bufp)
 			return (-1);
-		sop = (u_short *)bufp->page;
+		sop = (u_int16_t *)bufp->page;
 
 		if (PAIRFITS(sop, key, val))
 			putpair((char *)sop, key, val);
@@ -487,12 +489,12 @@ __add_ovflpage(hashp, bufp)
 	HTAB *hashp;
 	BUFHEAD *bufp;
 {
-	register u_short *sp;
-	u_short ndx, ovfl_num;
+	u_int16_t *sp;
+	u_int16_t ndx, ovfl_num;
 #ifdef DEBUG1
 	int tmp1, tmp2;
 #endif
-	sp = (u_short *)bufp->page;
+	sp = (u_int16_t *)bufp->page;
 
 	/* Check if we are dynamically determining the fill factor */
 	if (hashp->FFACTOR == DEF_FFACTOR) {
@@ -539,12 +541,12 @@ extern int
 __get_page(hashp, p, bucket, is_bucket, is_disk, is_bitmap)
 	HTAB *hashp;
 	char *p;
-	u_int bucket;
+	u_int32_t bucket;
 	int is_bucket, is_disk, is_bitmap;
 {
-	register int fd, page, size;
+	int fd, page, size;
 	int rsize;
-	u_short *bp;
+	u_int16_t *bp;
 
 	fd = hashp->fp;
 	size = hashp->BSIZE;
@@ -560,7 +562,7 @@ __get_page(hashp, p, bucket, is_bucket, is_disk, is_bitmap)
 	if ((lseek(fd, (off_t)page << hashp->BSHIFT, SEEK_SET) == -1) ||
 	    ((rsize = read(fd, p, size)) == -1))
 		return (-1);
-	bp = (u_short *)p;
+	bp = (u_int16_t *)p;
 	if (!rsize)
 		bp[0] = 0;	/* We hit the EOF, so initialize a new page */
 	else
@@ -572,12 +574,12 @@ __get_page(hashp, p, bucket, is_bucket, is_disk, is_bitmap)
 		PAGE_INIT(p);
 	} else
 		if (hashp->LORDER != BYTE_ORDER) {
-			register int i, max;
+			int i, max;
 
 			if (is_bitmap) {
 				max = hashp->BSIZE >> 2; /* divide by 4 */
 				for (i = 0; i < max; i++)
-					M_32_SWAP(((long *)p)[i]);
+					M_32_SWAP(((int *)p)[i]);
 			} else {
 				M_16_SWAP(bp[0]);
 				max = bp[0] + 2;
@@ -599,10 +601,10 @@ extern int
 __put_page(hashp, p, bucket, is_bucket, is_bitmap)
 	HTAB *hashp;
 	char *p;
-	u_int bucket;
+	u_int32_t bucket;
 	int is_bucket, is_bitmap;
 {
-	register int fd, page, size;
+	int fd, page, size;
 	int wsize;
 
 	size = hashp->BSIZE;
@@ -611,17 +613,17 @@ __put_page(hashp, p, bucket, is_bucket, is_bitmap)
 	fd = hashp->fp;
 
 	if (hashp->LORDER != BYTE_ORDER) {
-		register int i;
-		register int max;
+		int i;
+		int max;
 
 		if (is_bitmap) {
 			max = hashp->BSIZE >> 2;	/* divide by 4 */
 			for (i = 0; i < max; i++)
-				M_32_SWAP(((long *)p)[i]);
+				M_32_SWAP(((int *)p)[i]);
 		} else {
-			max = ((u_short *)p)[0] + 2;
+			max = ((u_int16_t *)p)[0] + 2;
 			for (i = 0; i <= max; i++)
-				M_16_SWAP(((u_short *)p)[i]);
+				M_16_SWAP(((u_int16_t *)p)[i]);
 		}
 	}
 	if (is_bucket)
@@ -645,14 +647,14 @@ __put_page(hashp, p, bucket, is_bucket, is_bitmap)
  * once they are read in.
  */
 extern int
-__init_bitmap(hashp, pnum, nbits, ndx)
+__ibitmap(hashp, pnum, nbits, ndx)
 	HTAB *hashp;
 	int pnum, nbits, ndx;
 {
-	u_long *ip;
+	u_int32_t *ip;
 	int clearbytes, clearints;
 
-	if ((ip = (u_long *)malloc(hashp->BSIZE)) == NULL)
+	if ((ip = (u_int32_t *)malloc(hashp->BSIZE)) == NULL)
 		return (1);
 	hashp->nmaps++;
 	clearints = ((nbits - 1) >> INT_BYTE_SHIFT) + 1;
@@ -662,16 +664,16 @@ __init_bitmap(hashp, pnum, nbits, ndx)
 	    hashp->BSIZE - clearbytes);
 	ip[clearints - 1] = ALL_SET << (nbits & BYTE_MASK);
 	SETBIT(ip, 0);
-	hashp->BITMAPS[ndx] = (u_short)pnum;
+	hashp->BITMAPS[ndx] = (u_int16_t)pnum;
 	hashp->mapp[ndx] = ip;
 	return (0);
 }
 
-static u_long
+static u_int32_t
 first_free(map)
-	u_long map;
+	u_int32_t map;
 {
-	register u_long i, mask;
+	u_int32_t i, mask;
 
 	mask = 0x1;
 	for (i = 0; i < BITS_PER_MAP; i++) {
@@ -682,13 +684,13 @@ first_free(map)
 	return (i);
 }
 
-static u_short
+static u_int16_t
 overflow_page(hashp)
 	HTAB *hashp;
 {
-	register u_long *freep;
-	register int max_free, offset, splitnum;
-	u_short addr;
+	u_int32_t *freep;
+	int max_free, offset, splitnum;
+	u_int16_t addr;
 	int bit, first_page, free_bit, free_page, i, in_use_bits, j;
 #ifdef DEBUG2
 	int tmp1, tmp2;
@@ -702,14 +704,14 @@ overflow_page(hashp)
 	/* Look through all the free maps to find the first free block */
 	first_page = hashp->LAST_FREED >>(hashp->BSHIFT + BYTE_SHIFT);
 	for ( i = first_page; i <= free_page; i++ ) {
-		if (!(freep = (u_long *)hashp->mapp[i]) &&
+		if (!(freep = (u_int32_t *)hashp->mapp[i]) &&
 		    !(freep = fetch_bitmap(hashp, i)))
-			return (NULL);
+			return (0);
 		if (i == free_page)
 			in_use_bits = free_bit;
 		else
 			in_use_bits = (hashp->BSIZE << BYTE_SHIFT) - 1;
-		
+
 		if (i == first_page) {
 			bit = hashp->LAST_FREED &
 			    ((hashp->BSIZE << BYTE_SHIFT) - 1);
@@ -734,7 +736,7 @@ overflow_page(hashp)
 	if (offset > SPLITMASK) {
 		if (++splitnum >= NCACHED) {
 			(void)write(STDERR_FILENO, OVMSG, sizeof(OVMSG) - 1);
-			return (NULL);
+			return (0);
 		}
 		hashp->OVFL_POINT = splitnum;
 		hashp->SPARES[splitnum] = hashp->SPARES[splitnum-1];
@@ -747,7 +749,7 @@ overflow_page(hashp)
 		free_page++;
 		if (free_page >= NCACHED) {
 			(void)write(STDERR_FILENO, OVMSG, sizeof(OVMSG) - 1);
-			return (NULL);
+			return (0);
 		}
 		/*
 		 * This is tricky.  The 1 indicates that you want the new page
@@ -760,9 +762,9 @@ overflow_page(hashp)
 		 * don't have to if we tell init_bitmap not to leave it clear
 		 * in the first place.
 		 */
-		if (__init_bitmap(hashp, (int)OADDR_OF(splitnum, offset),
-		    1, free_page))
-			return (NULL);
+		if (__ibitmap(hashp,
+		    (int)OADDR_OF(splitnum, offset), 1, free_page))
+			return (0);
 		hashp->SPARES[splitnum]++;
 #ifdef DEBUG2
 		free_bit = 2;
@@ -772,7 +774,7 @@ overflow_page(hashp)
 			if (++splitnum >= NCACHED) {
 				(void)write(STDERR_FILENO, OVMSG,
 				    sizeof(OVMSG) - 1);
-				return (NULL);
+				return (0);
 			}
 			hashp->OVFL_POINT = splitnum;
 			hashp->SPARES[splitnum] = hashp->SPARES[splitnum-1];
@@ -816,7 +818,7 @@ found:
 	for (i = 0; (i < splitnum) && (bit > hashp->SPARES[i]); i++);
 	offset = (i ? bit - hashp->SPARES[i - 1] : bit);
 	if (offset >= SPLITMASK)
-		return (NULL);	/* Out of overflow pages */
+		return (0);	/* Out of overflow pages */
 	addr = OADDR_OF(i, offset);
 #ifdef DEBUG2
 	(void)fprintf(stderr, "OVERFLOW_PAGE: ADDR: %d BIT: %d PAGE %d\n",
@@ -835,16 +837,16 @@ __free_ovflpage(hashp, obufp)
 	HTAB *hashp;
 	BUFHEAD *obufp;
 {
-	register u_short addr;
-	u_long *freep;
+	u_int16_t addr;
+	u_int32_t *freep;
 	int bit_address, free_page, free_bit;
-	u_short ndx;
+	u_int16_t ndx;
 
 	addr = obufp->addr;
 #ifdef DEBUG1
 	(void)fprintf(stderr, "Freeing %d\n", addr);
 #endif
-	ndx = (((u_short)addr) >> SPLITSHIFT);
+	ndx = (((u_int16_t)addr) >> SPLITSHIFT);
 	bit_address =
 	    (ndx ? hashp->SPARES[ndx - 1] : 0) + (addr & SPLITMASK) - 1;
 	 if (bit_address < hashp->LAST_FREED)
@@ -900,11 +902,11 @@ open_temp(hashp)
  */
 static void
 squeeze_key(sp, key, val)
-	u_short *sp;
+	u_int16_t *sp;
 	const DBT *key, *val;
 {
-	register char *p;
-	u_short free_space, n, off, pageno;
+	char *p;
+	u_int16_t free_space, n, off, pageno;
 
 	p = (char *)sp;
 	n = sp[0];
@@ -925,14 +927,14 @@ squeeze_key(sp, key, val)
 	OFFSET(sp) = off;
 }
 
-static u_long *
+static u_int32_t *
 fetch_bitmap(hashp, ndx)
 	HTAB *hashp;
 	int ndx;
 {
 	if (ndx >= hashp->nmaps)
 		return (NULL);
-	if ((hashp->mapp[ndx] = (u_long *)malloc(hashp->BSIZE)) == NULL)
+	if ((hashp->mapp[ndx] = (u_int32_t *)malloc(hashp->BSIZE)) == NULL)
 		return (NULL);
 	if (__get_page(hashp,
 	    (char *)hashp->mapp[ndx], hashp->BITMAPS[ndx], 0, 1, 1)) {
