@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -25,12 +23,42 @@
 #if !defined(__ppc__)
 #include <stdint.h>
 #include <mach/clock.h>
+#include <mach/mach_time.h>
 
 extern mach_port_t clock_port;
 
-uint64_t mach_absolute_time(void) {
-	mach_timespec_t now;
-	(void)clock_get_time(clock_port, &now);
-	return (uint64_t)now.tv_sec * NSEC_PER_SEC + now.tv_nsec;
+inline static uint64_t
+fast_get_nano_from_abs(int scale)
+{
+    uint64_t value;
+    asm (
+        "rdtsc                  \n\t"
+        "movl   %%edx,%%esi     \n\t"
+        "mull   %%ecx           \n\t"
+        "movl   %%edx,%%edi     \n\t"
+        "movl   %%esi,%%eax     \n\t"
+        "mull   %%ecx           \n\t"
+        "xorl   %%ecx,%%ecx     \n\t"
+        "addl   %%edi,%%eax     \n\t"
+        "adcl   %%ecx,%%edx         "
+                : "=A"(value) : "c"(scale) : "%esi", "%edi");
+        return value;
+}
+
+uint64_t
+mach_absolute_time(void) {
+        static int scale = 0;
+
+        if (__builtin_expect(scale == 0, 0)) {
+                mach_timebase_info_data_t info;
+                mach_timebase_info(&info);
+                scale = info.numer;
+        }
+        if (__builtin_expect(scale == 1, 0)) {
+                mach_timespec_t now;
+                (void)clock_get_time(clock_port, &now);
+                return (uint64_t)now.tv_sec * NSEC_PER_SEC + now.tv_nsec;
+	}
+	return fast_get_nano_from_abs(scale);
 }
 #endif
