@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -35,38 +33,50 @@
 #include <sys/syscall.h>
 #include <unistd.h>
 
+#define	__APPLE_API_PRIVATE
+#include <machine/cpu_capabilities.h>
+#undef	__APPLE_API_PRIVATE
+
+
 int gettimeofday (struct timeval *tp, struct timezone *tzp)
 {
         static int validtz = 0;
         static struct timezone cached_tz = {0};
         struct timeval localtv;
-#ifdef __ppc__
-	extern __ppc_gettimeofday(struct timeval *, struct timezone *);
-#endif
   
-        if (tzp && (tp == NULL) && (validtz == 0)) {
-                tp = &localtv;
+        if (tp == NULL) {
+            if (tzp == NULL)
+                return	(0);
+            tp = &localtv;
         }
 
 #ifdef __ppc__
-	if(__ppc_gettimeofday(tp, tzp))
-		return(-1);
+        {
+            extern int __ppc_gettimeofday(struct timeval *, struct timezone *);
+            extern int __commpage_gettimeofday(struct timeval *);
+    
+            if (__commpage_gettimeofday(tp)) {		/* first try commpage */
+                if (__ppc_gettimeofday(tp,tzp)) {	/* if it fails, use syscall */
+                    return (-1);
+                }
+            }
+        }
 #else
         if (syscall (SYS_gettimeofday, tp, tzp) < 0) {
                 return (-1);
         }
 #endif
         if (tzp) {
-		if (validtz == 0)  {
-			struct tm *localtm = localtime ((time_t *)&tp->tv_sec);
-			cached_tz.tz_dsttime = localtm->tm_isdst;
-			cached_tz.tz_minuteswest =
-				(-localtm->tm_gmtoff / SECSPERMIN) +
-				(localtm->tm_isdst * MINSPERHOUR);
-			validtz = 1;
-		}
-		tzp->tz_dsttime = cached_tz.tz_dsttime;
-		tzp->tz_minuteswest = cached_tz.tz_minuteswest;
+            if (validtz == 0)  {
+                struct tm *localtm = localtime ((time_t *)&tp->tv_sec);
+                cached_tz.tz_dsttime = localtm->tm_isdst;
+                cached_tz.tz_minuteswest =
+                    (-localtm->tm_gmtoff / SECSPERMIN) +
+                    (localtm->tm_isdst * MINSPERHOUR);
+                validtz = 1;
+            }
+            tzp->tz_dsttime = cached_tz.tz_dsttime;
+            tzp->tz_minuteswest = cached_tz.tz_minuteswest;
         }
         return (0);
 }
